package com.roobo.svui_plugin

import android.content.Context
import android.content.pm.PackageManager
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import com.google.gson.Gson
import com.roobo.svui.SVuiSdk
import com.roobo.svui.SdkAccountInfo
import com.roobo.svui.SdkConstant
import com.roobo.svui.audio.IAudioSource
import com.roobo.svui.engine.EngineType
import com.roobo.svui.engine.cloud.AIConfigParams
import com.roobo.svui.error.ErrorCode
import com.roobo.svui.error.SVUIError
import com.roobo.svui.event.SVuiCloudCallBack
import com.roobo.svui.event.SVuiEventInfo
import com.roobo.svui.event.SVuiEventListener
import com.roobo.svui_plugin.bean.EvaluateResultBean
import com.roobo.svui_plugin.bean.QAResultBean
import com.roobo.svui_plugin.bean.SimpleResultBean
import com.roobo.svui_plugin.bean.TTSResultBean
import com.roobo.svui_plugin.utils.FileRecorder
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.io.File
import java.util.*


/** SvuiPlugin
 * 重要：所有 result.error中的第一个值必须为int
 * */
public class SvuiPlugin : FlutterPlugin, MethodCallHandler {
    private lateinit var context: Context
    private val TAG = "MyMethodHandler";
    private var pcmDirectory: String? = null
    private var wavDirectory: String? = null

    //    private val confPath = Environment.getExternalStorageDirectory().absolutePath +
//            "/roobo/svui/config.svui.json"
//    private val vuiSdPath = Environment.getExternalStorageDirectory().absolutePath +
//            "/roobo/svui"
    private val confPath = "config.svui.json"
    private val vuiSdPath = "/"
    private val mSceneList = ArrayList<String>()
    private var mAudioSource: IAudioSource? = null
    private val gson = Gson()
    private var isASRInitialized: Boolean = false
    private val mH = Handler(Looper.getMainLooper())

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext;
        initRecordDirectory()
        channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "svui_plugin")
        channel.setMethodCallHandler(this);
    }

    private fun initRecordDirectory() {
        pcmDirectory = context.getExternalFilesDir(null)?.absolutePath + File.separator + "pcm" + File.separator
        wavDirectory = context.getExternalFilesDir(null)?.absolutePath + File.separator + "wav" + File.separator
        val pcmFile = File(pcmDirectory)
        if (!pcmFile.exists()) {
            Log.d(TAG, "pcm directory doesn't exist")
            pcmFile.mkdirs()
        } else {
            clearPcm()
        }
        val wavFile = File(wavDirectory)
        if (!wavFile.exists()) {
            Log.d(TAG, "wav directory doesn't exist")
            wavFile.mkdirs()
        }
    }

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "svui_plugin")
            channel.setMethodCallHandler(SvuiPlugin())
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                success(result, "Android ${android.os.Build.VERSION.RELEASE}")
            }
            "initSVUI" -> {
                val accountInfo = gson.fromJson(call.argument<String>("accountInfo"), SdkAccountInfo::class.java)
                var isTest = false;
                if (null != call.argument<Boolean>("isTest")) {
                    isTest = call.argument<Boolean>("isTest")!!
                }
                initSVUI(result, accountInfo, isTest)
            }
            "oralEvaluate" -> {
                oralEvaluate(result, call.argument<String>("text"))
            }
            "tts" -> {
                tts(result, call.argument<String>("text"), call.argument<String>("language"))
            }
            "stopCapture" -> {
                stopCapture(result)
            }
            "release" -> {
                release()
            }
            "startQA" -> {
                startQA(result, call.argument<Int>("lessonId"),
                        call.argument<String>("dialogId"))
            }
            "clearAudioRecord" -> {
                clearAudioRecord(result)
            }
            "getAudioRecordSize" -> {
                getAudioRecordSize(result)
            }
            else -> {
                notImplemented(result)
            }
        }
    }

    @Synchronized
    private fun initSVUI(result: MethodChannel.Result, accountInfo: SdkAccountInfo, isTest: Boolean) {
        if (isASRInitialized) {
            Log.d(TAG, "already initialized");
            success(result, true)
            return
        }
        Thread(Runnable { // sdk 基本接口初始化
            SVuiSdk.getInstance().context = this.context
            SVuiSdk.getInstance().init(vuiSdPath)
//            GlobalConfig.SAVE_CLOUD_ASR_AUDIO = true
//            GlobalConfig.ASR_AUDIO_SAVE_DIR = audioPath
            SVuiSdk.getInstance().setCloudAudioSaveDir(pcmDirectory)
            SVuiSdk.getInstance().setLogLevel(SdkConstant.LogLevel.LOG_LEVEL_VERBOSE)
            if (isTest) {
                SVuiSdk.getInstance().setSdkEnv(SdkConstant.WorkEnv.ENV_TEST)
            } else {
                SVuiSdk.getInstance().setSdkEnv(SdkConstant.WorkEnv.ENV_ONLINE)
            }
            val ret = SVuiSdk.getInstance().setAccountInfo(accountInfo)
            Log.i(TAG, "setAccountInfo $ret")
            if (ret != ErrorCode.ERROR_OK) {
                Log.d(TAG, "init failed err:$ret")
                error(result, ret, "init failed", "init failed")
                return@Runnable
            }
            SVuiSdk.getInstance().reset(SVuiSdk.RESET_FULL)
            // 加载配置文件
            val loadResult = SVuiSdk.getInstance().loadConfig(confPath)
            if (loadResult != ErrorCode.ERROR_OK) {
                Log.d(TAG, "loadConfig failed err:$loadResult")
                error(result, loadResult, "init failed", "loadConfig failed")
                return@Runnable
            }
            mSceneList.clear()
            for (scene in SVuiSdk.getInstance().sceneList) {
                Log.d(TAG, "scene:$scene")
                mSceneList.add(scene)
            }
            isASRInitialized = true
            success(result, true)
        }).start()
    }

    private fun oralEvaluate(result: MethodChannel.Result, text: String?) {
        Log.d(TAG, "oralEvaluate: $text")
        if (TextUtils.isEmpty(text)) {
            return
        }
        if (!checkRecordPermissionGranted()) {
            error(result, OtherErrorCode.PERMISSION_DENIED, "record_audio permission denied",
                    "record_audio permission denied")
            return
        }
        //传入null停掉所有场景
        SVuiSdk.getInstance().switchToScenes(null)
        SVuiSdk.getInstance().startScene("oraltest")
        SVuiSdk.getInstance().setEngineParam(EngineType.CLOUD, AIConfigParams.AI_CONFIG_ORALTESTING_EX.name,
                "{\"text\": \"$text\"}")
        var pcmPath = ""
        var wavPath = ""
        SVuiSdk.getInstance().registerSVuiEventListener("oraltest", object : SVuiEventListener {
            override fun onEvent(i: Int, info: SVuiEventInfo) {
                Log.d(TAG, "onEvent eventType:$i ,info$info")
                val evaResult = gson.fromJson(info.text, EvaluateResultBean::class.java)
                if (TextUtils.isEmpty(pcmPath) || TextUtils.isEmpty(wavPath)) {
                    error(result, OtherErrorCode.RECORD_PATH_EMPTY, "audio record save failed", "audio record save failed")
                    return
                }
                FileRecorder.savePcmToWav(File(pcmPath), File(wavPath))
                FileRecorder.deleteFile(File(pcmPath))
                val simpleResult = SimpleResultBean(evaResult.Data?.pronunciation,
                        evaResult.Data?.words?.get(0)?.word,
                        wavPath)
                val simpleS = gson.toJson(simpleResult)
                mAudioSource?.stopCapture()
                success(result, simpleS)
            }

            override fun onSpeechEvent(i: Int) {
                Log.d(TAG, "onSpeechEvent eventType:$i")
            }

            override fun onCloudRecognizeStart(traceId: String?) {
                Log.d(TAG, "save file name:$traceId")
                pcmPath = "${pcmDirectory + traceId}.pcm"
                wavPath = "${wavDirectory + traceId}.wav"
            }

            override fun onError(error: SVUIError) {
                Log.d(TAG, "onError error:$error")
                mAudioSource?.stopCapture()
                error(result, error.code, error.message, error.detail)
            }
        })
        // 音频采集设置
        if (mAudioSource == null) {
            mAudioSource = SVuiSdk.getInstance().createAudioSource(SVuiSdk.AUDIO_SOURCE_ANDROID_DEFAULT)
        }
        if (!mAudioSource?.isCaptureStarted!!) {
            mAudioSource?.setAudioDataSink(SVuiSdk.getInstance().audioDataSink)
            val startSuccess = mAudioSource?.startCapture()
            if (!startSuccess!!) {
                error(result, OtherErrorCode.CAPTURE_FAILED,
                        "start capture failed", "start capture failed")
            }
        }
    }

    private fun startQA(result: MethodChannel.Result, lessonId: Int?, dialogId: String?) {
        Log.d(TAG, "startQA  lessonId:$lessonId     ,dialogId:$dialogId")
        if (null == lessonId || TextUtils.isEmpty(dialogId)) {
            return
        }
        if (!checkRecordPermissionGranted()) {
            error(result, OtherErrorCode.PERMISSION_DENIED, "record_audio permission denied",
                    "record_audio permission denied")
            return
        }
        //传入null停掉所有场景
        SVuiSdk.getInstance().switchToScenes(null)
        SVuiSdk.getInstance().startScene("voiceqa")
        var pcmPath = ""
        var wavPath = ""
        SVuiSdk.getInstance().setEngineParam(EngineType.CLOUD, AIConfigParams.AI_CONFIG_EXTRA.name,
                "[{" +
                        "    \"service\": \"RooboAPPCourse_QA\"," +
                        "    \"context\": \"\"," +
                        "    \"parameters\": {" +
                        "      \"grammarName\": {" +
                        "        \"orgin\": null," +
                        "        \"normType\": \"String\"," +
                        "        \"norm\": \"RooboAPPCourse_QA_${lessonId}_${dialogId}\"" +
                        "      }" +
                        "    }" +
                        "  }]")
        SVuiSdk.getInstance().registerSVuiEventListener("voiceqa", object : SVuiEventListener {
            override fun onEvent(i: Int, info: SVuiEventInfo) {
                mAudioSource?.stopCapture()
                if (TextUtils.isEmpty(pcmPath) || TextUtils.isEmpty(wavPath)) {
                    error(result, OtherErrorCode.RECORD_PATH_EMPTY, "audio record save failed", "audio record save failed")
                    return
                }
                FileRecorder.savePcmToWav(File(pcmPath), File(wavPath))
                FileRecorder.deleteFile(File(pcmPath))
                val qaResult = gson.fromJson(info.text.replace("////".toRegex(), ""), QAResultBean::class.java)
                qaResult.recordPath = wavPath
                val simpleS = gson.toJson(qaResult)
                Log.d(TAG, "QA:$simpleS")
                success(result, simpleS)
            }

            override fun onSpeechEvent(i: Int) {
                Log.d(TAG, "onSpeechEvent eventType:$i")
            }

            override fun onCloudRecognizeStart(traceId: String?) {
                Log.d(TAG, "save file name:$traceId")
                pcmPath = "${pcmDirectory + traceId}.pcm"
                wavPath = "${wavDirectory + traceId}.wav"
            }

            override fun onError(error: SVUIError) {
                Log.d(TAG, "onError error:$error")
                mAudioSource?.stopCapture()
                error(result, error.code, error.message, error.detail)
            }
        })
        // 音频采集设置
        if (mAudioSource == null) {
            mAudioSource = SVuiSdk.getInstance().createAudioSource(SVuiSdk.AUDIO_SOURCE_ANDROID_DEFAULT)
        }
        if (!mAudioSource?.isCaptureStarted!!) {
            mAudioSource?.setAudioDataSink(SVuiSdk.getInstance().audioDataSink)
            val startSuccess = mAudioSource?.startCapture()
            if (!startSuccess!!) {
                error(result, OtherErrorCode.CAPTURE_FAILED,
                        "start capture failed", "start capture failed")
            }
        }
    }

    private fun checkRecordPermissionGranted(): Boolean {
        val packageManager: PackageManager = context.packageManager
        val packageName = context.packageName
        return PackageManager.PERMISSION_DENIED != packageManager.checkPermission(
                "android.permission.RECORD_AUDIO", packageName)
    }

    private fun stopCapture(result: MethodChannel.Result) {
        if (null == mAudioSource) {
            error(result, OtherErrorCode.NULL_POINTER, "audio source is null", "audio source is null")
            return
        }
        mAudioSource?.stopCapture();
    }

    private fun tts(result: MethodChannel.Result, text: String?, language: String?) {
        Log.d(TAG, "tts:$text")
        SVuiSdk.getInstance().setSVuiCloudCallBack(object : SVuiCloudCallBack {
            override fun onCloudResult(p0: Int, p1: String?) {
                val ttsResult = gson.fromJson(p1, TTSResultBean::class.java)
                if (p0 == ErrorCode.ERROR_OK) {
                    success(result, ttsResult?.tts?.content)
                } else {
                    error(result, p0, ttsResult?.status?.errorType, ttsResult?.status?.errorType)
                }
            }
        })
        Thread(Runnable {
            if (TextUtils.isEmpty(language)) {
                SVuiSdk.getInstance().ttsTransform(text)
            } else {
                SVuiSdk.getInstance().ttsTransform(text, language)
            }
        }).start()
    }

    private fun clearAudioRecord(result: Result) {
        FileRecorder.clearDirectory(File(wavDirectory))
        result.success(true)
    }

    private fun getAudioRecordSize(result: Result) {
        val file = File(wavDirectory)
        val size = FileRecorder.getFolderSize(file)
        success(result, size)
    }

    private fun clearPcm() {
        FileRecorder.clearDirectory(File(pcmDirectory))
    }

    private fun success(result: MethodChannel.Result, obj: Any?) {
        mH.post(Runnable {
            result.success(obj)
        })
    }

    private fun error(result: MethodChannel.Result, code: Int, message: String?, detail: String?) {
        mH.post(Runnable {
            result.error(code.toString(), message, detail)
        })
    }

    private fun notImplemented(result: MethodChannel.Result) {
        mH.post(Runnable {
            result.notImplemented()
        })
    }

    private fun release() {
        //TODO release svuisdk
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
