import 'dart:async';
import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:svui_plugin/account.dart';
import 'package:svui_plugin/config.dart';
import 'package:svui_plugin/error.dart';
import 'package:svui_plugin/svui/evaluatebean.dart';
import 'package:svui_plugin/svui/qaresult.dart';
import 'package:svui_plugin/svui/svui_listener.dart';
import 'package:svui_plugin/tts/PlayerException.dart';
import 'package:svui_plugin/tts/iplayer.dart';
import 'package:svui_plugin/tts/itts.dart';
import 'package:svui_plugin/tts/ttslistener.dart';

import 'svui/svui_listener.dart';

class SvuiPlugin {
  ITTS __ttsPlayer = _TTSPlayer();

  SvuiPlugin._();

  static SvuiPlugin _instance;

  static SvuiPlugin getInstance() {
    if (_instance == null) {
      _instance = SvuiPlugin._();
    }
    return _instance;
  }

  Future<String> get platformVersion async {
    final String version =
        await Configure.channel.invokeMethod('getPlatformVersion');
    return version;
  }

  void init(SdkAccountInfo info,
      {SVUIInitListener listener, bool isTest, IPlayer player}) async {
    if (null != player) {
      setTTSPlayer(player);
    }
    try {
      String infoJson = json.encode(info);
      bool res = await Configure.channel.invokeMethod("initSVUI",
          {'accountInfo': infoJson, 'isTest': null == isTest ? false : isTest});
      if (res) {
        if (null != listener) {
          listener.onCompleted();
        }
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  void startOralEvaluate(String text, OralEvluateListener listener) async {
    try {
      //result.success(any)
      String res =
          await Configure.channel.invokeMethod('oralEvaluate', {'text': text});

      EvaluateResultBean resultBean =
          EvaluateResultBean.fromJson(json.decode(res));
      if (null != listener) {
        listener.onOralEvaluateResult(resultBean);
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onOralEvaluateFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onOralEvaluateFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  void startQA(int lessionId, String dialogId, QAListener listener) async {
    try {
      //result.success(any)
      String res = await Configure.channel.invokeMethod(
          'startQA', {'lessonId': lessionId, 'dialogId': dialogId});
      QAResultBean resultBean = QAResultBean.fromJson(json.decode(res));
      if (null != listener) {
        listener.onQAResult(
            res, resultBean.ai.semantic.action, resultBean.recordPath);
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onQAFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onQAFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  void stopCapture({SVUIStopListener listener}) {
    try {
      Configure.channel.invokeMethod("stopCapture");
      if (null != listener) {
        listener.onStopSuccess();
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onStopFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onStopFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  void release() {
    try {
      __ttsPlayer.release();
      Configure.channel.invokeMethod("release");
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
    }
  }

  void speechSynthesis(String text,
      {String language, SpeechSynthesisListener listener}) async {
    try {
      //result.success(any)
      String url = await Configure.channel
          .invokeMethod('tts', {'text': text, 'language': language});
      if (null != listener) {
        listener.onSpeechSynthesisSuccess(url);
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onSpeechSynthesisFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onSpeechSynthesisFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  void speak(String text, {String language, PlayListener listener}) {
    __ttsPlayer.speak(text, listener: listener);
  }

  void stopSpeak() {
    __ttsPlayer.stop();
  }

  bool isSpeaking() {
    return __ttsPlayer.isSpeaking();
  }

  void setTTSPlayer(IPlayer player) {
    __ttsPlayer.setPlayer(player);
  }
}

// TODO CACHE
class _TTSPlayer implements ITTS {
  static const String _TAG = "TTSPlayerDebug";
  IPlayer _mPlayer;

  @override
  bool isSpeaking() {
    if (null == _mPlayer) {
      return false;
    }
    return _mPlayer.isPlaying();
  }

  @override
  Future speak(String text, {String language, PlayListener listener}) async {
    try {
      //result.success(any)
      String url = await Configure.channel
          .invokeMethod('tts', {'text': text, 'language': language});
      if (null != _mPlayer) {
        _mPlayer.playUrl(url, listener: listener);
      } else {
        throw PlayerException(
            "before invoke speak method please call setPlayer first");
      }
    } on PlatformException catch (e) {
      // result.error("failed", "failed unknow", "i don't know")
      print(e);
      if (null != listener) {
        listener.onFailed(int.parse(e.code), e.message);
      }
    } on MissingPluginException catch (e) {
      //result.notImplemented()
      print(e);
      if (null != listener) {
        listener.onFailed(ErrorCode.NOT_IMPLEMENTED, e.message);
      }
    }
  }

  @override
  void setPlayer(IPlayer player) {
    if (null != _mPlayer) {
      _mPlayer.release();
    }
    _mPlayer = player;
    _mPlayer.init();
  }

  @override
  void stop() {
    if (null != _mPlayer) {
      _mPlayer.stop();
    }
  }

  @override
  void release() {
    if (null != _mPlayer) {
      _mPlayer.release();
    }
  }
}
